--[[
	StopFullCombine.lua
	
	Autor: 		Ifko[nator]
	Datum: 		11.04.2022
	Version: 	1.3
	
	Changelog:	v1.0 @31.03.2022 - initial implementation in FS 22
				---------------------------------------------------------------
				v1.1 @01.04.2022 - stopped straw swath, when combine is stopped
				---------------------------------------------------------------
				v1.2 @03.04.2022 - pipe will now fold in, when combine is empty
				---------------------------------------------------------------
				v1.3 @11.04.2022 - fix for attached combines
]]

StopFullCombine = {};

function StopFullCombine.prerequisitesPresent(specializations)
	return SpecializationUtil.hasSpecialization(Combine, specializations);
end;

function StopFullCombine.registerFunctions(vehicleType)
	local newFunctions = {
		"lockCombine",
		"resetTimer",
		"resetStrawTimer",
		"startBeaconLights",
		"stopBeaconLights"
	};
	
	for _, newFunction in ipairs(newFunctions) do
		SpecializationUtil.registerFunction(vehicleType, newFunction, StopFullCombine[newFunction]);
	end;
end;

function StopFullCombine.registerOverwrittenFunctions(vehicleType)
	local overwrittenFunctions = {
		"processCombineSwathArea"
	};
	
	for _, overwrittenFunction in ipairs(overwrittenFunctions) do
		SpecializationUtil.registerOverwrittenFunction(vehicleType, overwrittenFunction, StopFullCombine[overwrittenFunction]);
	end;
end;

function StopFullCombine.registerEventListeners(vehicleType)
	local functionNames = {
		"onLoad",
		"onUpdate"
	};
	
	for _, functionName in ipairs(functionNames) do
		SpecializationUtil.registerEventListener(vehicleType, functionName, StopFullCombine);
	end;
end;

function StopFullCombine:onLoad(savegame)
	local specCombine = self.spec_combine;
	
	self:resetTimer(specCombine);
	self:resetStrawTimer(specCombine);

	specCombine.stopTimerMax = 5000; --## stop combine for 5 seconds
	specCombine.strawStopTimerMax = 500; --## stop straw for 0.5 seconds
	specCombine.needTurnOnBeaconLights = true;
	specCombine.beaconLightsWasTurnedOn = false;
	specCombine.hasTurnedOffSwath = false;
	specCombine.allowChangePipeState = false;
end;

function StopFullCombine:onUpdate(dt, isActiveForInput, isActiveForInputIgnoreSelection, isSelected)
	if self:getIsActiveForInput(true, false) then
		local specCombine = self.spec_combine;
		local specLights = self.spec_lights;
		local specDrivable = self.spec_drivable;
		local isMotorStarted = self.getIsMotorStarted ~= nil and self:getIsMotorStarted();
		local controlledTractor = nil;
		
		if self.getAttacherVehicle ~= nil then
			controlledTractor = self:getAttacherVehicle();
		end;

		if controlledTractor ~= nil then
			specDrivable = controlledTractor.spec_drivable;
			isMotorStarted = controlledTractor.getIsMotorStarted ~= nil and controlledTractor:getIsMotorStarted();
		end;
		
		if specDrivable ~= nil and specCombine ~= nil and isMotorStarted then
			if 	specLights ~= nil and #specLights.beaconLights > 0 then
				if self:getFillUnitFillLevelPercentage(specCombine.fillUnitIndex) >= 0.9 then
					self:startBeaconLights(specCombine.needTurnOnBeaconLights, specCombine);
				else
					self:stopBeaconLights(specCombine.beaconLightsWasTurnedOn, specCombine);
				end;
			end;

			if not specCombine.isBufferCombine then
				if self:getFillUnitFreeCapacity(specCombine.fillUnitIndex) == 0 then
					if specCombine.stopTimer < specCombine.stopTimerMax then
						self:lockCombine(self.spec_wheels, self.spec_pipe, specCombine, controlledTractor);

						specCombine.stopTimer = specCombine.stopTimer + dt;
					end;

					if specCombine.strawStopTimer < specCombine.strawStopTimerMax then
						specCombine.strawStopTimer = specCombine.strawStopTimer + dt;
					end;
				elseif self:getFillUnitFreeCapacity(specCombine.fillUnitIndex) == self:getFillUnitCapacity(specCombine.fillUnitIndex) then
					if specCombine.allowChangePipeState then
						--## fold pipe in, when empty

						self:setPipeState(1);

						specCombine.allowChangePipeState = false;
					end;
				else
					if specCombine.stopTimer > 0 then
						self:resetTimer(specCombine);
					end;

					if specCombine.strawStopTimer > 0 then
						self:resetStrawTimer(specCombine);
					end;
				end;
			end;

			specCombine.hasTurnedOffSwath = specCombine.strawStopTimer >= specCombine.strawStopTimerMax;
		end;
	end;
end;

function StopFullCombine:resetTimer(specCombine)
	specCombine.stopTimer = 0;
end;

function StopFullCombine:resetStrawTimer(specCombine)
	specCombine.strawStopTimer = 0;
end;

function StopFullCombine:lockCombine(specWheels, specPipe, specCombine, controlledTractor)
	if self.isServer then
		for _, wheel in pairs(specWheels.wheels) do
			setWheelShapeProps(wheel.node, wheel.wheelShape, 0, math.huge, 0, 0);
		end;

		if controlledTractor ~= nil then
			for _, wheel in pairs(controlledTractor.spec_wheels.wheels) do
				setWheelShapeProps(wheel.node, wheel.wheelShape, 0, math.huge, 0, 0);
			end;
		end;
	end;

	self:setBrakeLightsVisibility(true);
	
	if self.getCruiseControlState ~= nil and self:getCruiseControlState() ~= Drivable.CRUISECONTROL_STATE_OFF then
		self:setCruiseControlState(Drivable.CRUISECONTROL_STATE_OFF);
	end;

	if controlledTractor ~= nil and controlledTractor.getCruiseControlState ~= nil and controlledTractor:getCruiseControlState() ~= Drivable.CRUISECONTROL_STATE_OFF then
		controlledTractor:setCruiseControlState(Drivable.CRUISECONTROL_STATE_OFF);
	end;

	if specPipe ~= nil and specPipe.hasMovablePipe then
		if specPipe.targetState == 1 then
			--## unfold pipe, when full

			self:setPipeState(next(specPipe.unloadingStates));
		end;
		
		specCombine.allowChangePipeState = true;
	end;
end;

function StopFullCombine:startBeaconLights(needTurnOnBeaconLights, specCombine)
	if needTurnOnBeaconLights then
		self:setBeaconLightsVisibility(true, true, nil);

		specCombine.needTurnOnBeaconLights = false;
		specCombine.beaconLightsWasTurnedOn = true;
	end;
end;

function StopFullCombine:stopBeaconLights(beaconLightsWasTurnedOn, specCombine)
	if beaconLightsWasTurnedOn then
		self:setBeaconLightsVisibility(false, true, nil);

		specCombine.needTurnOnBeaconLights = true;
		specCombine.beaconLightsWasTurnedOn = false;
	end;
end;

function StopFullCombine:processCombineSwathArea(superFunc, workArea)
	local specCombine = self.spec_combine;
	
	if specCombine.hasTurnedOffSwath then
		specCombine.processing.inputBuffer.buffer[specCombine.processing.inputBuffer.dropIndex].liters = 0;
		specCombine.workAreaParameters.litersToDrop = 0;
	end;
	
	local litersToDrop = specCombine.workAreaParameters.litersToDrop;
	
	if specCombine.isSwathActive and litersToDrop > 0 then
		local droppedLiters = 0;
		local fruitDesc = g_fruitTypeManager:getFruitTypeByFillTypeIndex(specCombine.workAreaParameters.dropFillType);

		if fruitDesc ~= nil and fruitDesc.windrowLiterPerSqm ~= nil then
			local windrowFillType = g_fruitTypeManager:getWindrowFillTypeIndexByFruitTypeIndex(fruitDesc.index);

			if windrowFillType ~= nil then
				local sx, sy, sz, ex, ey, ez = DensityMapHeightUtil.getLineByArea(workArea.start, workArea.width, workArea.height, true);
				local dropped, lineOffset = DensityMapHeightUtil.tipToGroundAroundLine(self, litersToDrop, windrowFillType, sx, sy, sz, ex, ey, ez, 0, nil, workArea.lineOffset, false, nil, false);
				
				droppedLiters = dropped;
				
				workArea.lineOffset = lineOffset;
			end;
		end;

		if droppedLiters > 0 then
			specCombine.workAreaParameters.isStrawEffectEnabled = 500;
		end;

		specCombine.workAreaParameters.droppedLiters = droppedLiters;
	end;

	return specCombine.workAreaParameters.lastRealArea, specCombine.workAreaParameters.lastArea;
end;